@@ -13,6 +13,11 @@
 #include <fcitx/userinterface.h>
 #include "fcitxcore_export.h"
 
+/// \addtogroup FcitxCore
+/// \{
+/// \file
+/// \brief Input Method event for Fcitx.
+
 namespace fcitx {
 
 class InputContext;
@@ -47,6 +52,8 @@ enum class InputMethodSwitchedReason {
     Other,
 };
 
+enum class InputMethodMode { PhysicalKeyboard, OnScreenKeyboard };
+
 /**
  * Type of input method events.
  */
@@ -80,7 +87,7 @@ enum class EventType : uint32_t {
      *
      * So OnClose is called when preedit IS committed (not like
      * CChangeByInactivate,
-     * this behavior cannot be overrided), it give im a chance to choose
+     * this behavior cannot be overridden), it give im a chance to choose
      * remember this
      * word or not.
      *
@@ -126,6 +133,8 @@ enum class EventType : uint32_t {
      */
     InputContextInvokeAction = InputContextEventFlag | 0xE,
 
+    InputContextVirtualKeyboardEvent = InputContextEventFlag | 0xF,
+
     InputContextForwardKey = InputMethodEventFlag | 0x1,
     InputContextCommitString = InputMethodEventFlag | 0x2,
     InputContextDeleteSurroundingText = InputMethodEventFlag | 0x3,
@@ -168,6 +177,26 @@ enum class EventType : uint32_t {
      * @see FocusOutEvent
      */
     FocusGroupFocusChanged = InstanceEventFlag | 0x5,
+    /**
+     * Input method mode changed
+     */
+    InputMethodModeChanged = InstanceEventFlag | 0x6,
+    /**
+     * Global config is reloaded
+     *
+     * This only fires after fcitx has entered running state.
+     * The initial load will not trigger this event.
+     * @see GlobalConfig
+     * @since 5.1.0
+     */
+    GlobalConfigReloaded = InstanceEventFlag | 0x7,
+    /**
+     * Virtual keyboard visibility changed
+     *
+     * @see UserInterfaceManager
+     * @since 5.1.0
+     */
+    VirtualKeyboardVisibilityChanged = InstanceEventFlag | 0x8,
 };
 
 /**
@@ -327,6 +356,38 @@ private:
     bool filtered_ = false;
 };
 
+class VirtualKeyboardEventPrivate;
+
+class FCITXCORE_EXPORT VirtualKeyboardEvent : public InputContextEvent {
+public:
+    VirtualKeyboardEvent(InputContext *context, bool isRelease, int time = 0);
+    ~VirtualKeyboardEvent();
+
+    int time() const;
+
+    void setKey(Key key);
+    const Key &key() const;
+
+    void setPosition(float x, float y);
+    float x() const;
+    float y() const;
+
+    void setLongPress(bool longPress);
+    bool isLongPress() const;
+
+    void setUserAction(uint64_t actionId);
+    uint64_t userAction() const;
+
+    void setText(std::string text);
+    const std::string &text() const;
+
+    std::unique_ptr<KeyEvent> toKeyEvent() const;
+
+protected:
+    FCITX_DECLARE_PRIVATE(VirtualKeyboardEvent);
+    std::unique_ptr<VirtualKeyboardEventPrivate> d_ptr;
+};
+
 class FCITXCORE_EXPORT ForwardKeyEvent : public KeyEventBase {
 public:
     ForwardKeyEvent(InputContext *context, Key rawKey, bool isRelease = false,
@@ -416,6 +477,12 @@ protected:
     bool immediate_;
 };
 
+class FCITXCORE_EXPORT VirtualKeyboardVisibilityChangedEvent : public Event {
+public:
+    VirtualKeyboardVisibilityChangedEvent()
+        : Event(EventType::VirtualKeyboardVisibilityChanged) {}
+};
+
 class FCITXCORE_EXPORT InputMethodNotificationEvent : public InputContextEvent {
 public:
     InputMethodNotificationEvent(EventType type, const std::string &name,
@@ -514,6 +581,28 @@ private:
     InputContext *newFocus_;
 };
 
+/**
+ * Notify the input method mode is changed.
+ *
+ * @see Instance::InputMethodMode
+ * @since 5.1.0
+ */
+class FCITXCORE_EXPORT InputMethodModeChangedEvent : public Event {
+public:
+    InputMethodModeChangedEvent() : Event(EventType::InputMethodModeChanged) {}
+};
+
+/**
+ * Notify the global config is reloaded.
+ *
+ * @see GlobalConfig
+ * @since 5.1.0
+ */
+class FCITXCORE_EXPORT GlobalConfigReloadedEvent : public Event {
+public:
+    GlobalConfigReloadedEvent() : Event(EventType::GlobalConfigReloaded) {}
+};
+
 class FCITXCORE_EXPORT CapabilityEvent : public InputContextEvent {
 public:
     CapabilityEvent(InputContext *ic, EventType type, CapabilityFlags oldFlags,
@@ -544,6 +633,7 @@ public:
         : CapabilityEvent(ic, EventType::InputContextCapabilityAboutToChange,
                           oldFlags, newFlags) {}
 };
+
 } // namespace fcitx
 
 #endif // _FCITX_EVENT_H_
