@@ -137,6 +137,8 @@ public:
     InputPanel inputPanel_;
     StatusArea statusArea_;
     bool hasFocus_ = false;
+    bool clientControlVirtualkeyboardShow_ = false;
+    bool clientControlVirtualkeyboardHide_ = false;
     std::string program_;
     CapabilityFlags capabilityFlags_;
     bool isPreeditEnabled_ = true;
@@ -152,6 +154,7 @@ public:
 
     std::list<std::unique_ptr<InputContextEvent>> blockedEvents_;
     bool blockEventToClient_ = false;
+    bool lastPreeditUpdateIsEmpty_ = true;
 };
 } // namespace fcitx
 
