@@ -18,9 +18,11 @@
 #include "fcitx-utils/event.h"
 #include "fcitx-utils/i18n.h"
 #include "fcitx-utils/log.h"
+#include "fcitx-utils/misc.h"
 #include "fcitx-utils/standardpath.h"
 #include "fcitx-utils/stringutils.h"
 #include "fcitx-utils/utf8.h"
+#include "fcitx/event.h"
 #include "../../modules/notifications/notifications_public.h"
 #include "addonmanager.h"
 #include "focusgroup.h"
@@ -429,7 +431,8 @@ bool InstancePrivate::canDeactivate(InputContext *ic) {
     return inputState->isActive();
 }
 
-void InstancePrivate::navigateGroup(InputContext *ic, bool forward) {
+void InstancePrivate::navigateGroup(InputContext *ic, const Key &key,
+                                    bool forward) {
     auto *inputState = ic->propertyFor(&inputStateFactory_);
     inputState->pendingGroupIndex_ =
         (inputState->pendingGroupIndex_ +
@@ -437,7 +440,7 @@ void InstancePrivate::navigateGroup(InputContext *ic, bool forward) {
         imManager_.groupCount();
     FCITX_DEBUG() << "Switch to group " << inputState->pendingGroupIndex_;
 
-    if (notifications_) {
+    if (notifications_ && !isSingleKey(key)) {
         notifications_->call<INotifications::showTip>(
             "enumerate-group", _("Input Method"), "input-keyboard",
             _("Switch group"),
@@ -447,13 +450,17 @@ void InstancePrivate::navigateGroup(InputContext *ic, bool forward) {
     }
 }
 
-void InstancePrivate::acceptGroupChange(InputContext *ic) {
+void InstancePrivate::acceptGroupChange(const Key &key, InputContext *ic) {
     FCITX_DEBUG() << "Accept group change";
 
     auto *inputState = ic->propertyFor(&inputStateFactory_);
     auto groups = imManager_.groups();
     if (groups.size() > inputState->pendingGroupIndex_) {
-        imManager_.setCurrentGroup(groups[inputState->pendingGroupIndex_]);
+        if (isSingleKey(key)) {
+            imManager_.enumerateGroupTo(groups[inputState->pendingGroupIndex_]);
+        } else {
+            imManager_.setCurrentGroup(groups[inputState->pendingGroupIndex_]);
+        }
     }
     inputState->pendingGroupIndex_ = 0;
 }
@@ -709,6 +716,7 @@ Instance::Instance(int argc, char **argv) {
             auto &keyEvent = static_cast<KeyEvent &>(event);
             auto *ic = keyEvent.inputContext();
             CheckInputMethodChanged imChangedRAII(ic, d);
+            auto origKey = keyEvent.origKey().normalize();
 
             struct {
                 const KeyList &list;
@@ -737,10 +745,14 @@ Instance::Instance(int argc, char **argv) {
                  [this, ic](bool) { return enumerate(ic, false); }},
                 {d->globalConfig_.enumerateGroupForwardKeys(),
                  [this]() { return canChangeGroup(); },
-                 [ic, d](bool) { return d->navigateGroup(ic, true); }},
+                 [ic, d, origKey](bool) {
+                     return d->navigateGroup(ic, origKey, true);
+                 }},
                 {d->globalConfig_.enumerateGroupBackwardKeys(),
                  [this]() { return canChangeGroup(); },
-                 [ic, d](bool) { return d->navigateGroup(ic, false); }},
+                 [ic, d, origKey](bool) {
+                     return d->navigateGroup(ic, origKey, false);
+                 }},
             };
 
             auto *inputState = ic->propertyFor(&d->inputStateFactory_);
@@ -749,15 +761,9 @@ Instance::Instance(int argc, char **argv) {
             // Keep these two values, and reset them in the state
             inputState->keyReleased_ = -1;
             inputState->lastKeyPressed_ = Key();
-            auto origKey = keyEvent.origKey().normalize();
             const bool isModifier = origKey.isModifier();
             if (keyEvent.isRelease()) {
                 int idx = 0;
-                if (origKey.isModifier() &&
-                    (Key::keySymToStates(origKey.sym()) == origKey.states() ||
-                     origKey.states() == 0)) {
-                    inputState->totallyReleased_ = true;
-                }
                 for (auto &keyHandler : keyHandlers) {
                     if (keyReleased == idx &&
                         origKey.isReleaseOfModifier(lastKeyPressed) &&
@@ -768,10 +774,14 @@ Instance::Instance(int argc, char **argv) {
                                 inputState->totallyReleased_ = false;
                             }
                         }
-                        return keyEvent.filter();
+                        keyEvent.filter();
+                        break;
                     }
                     idx++;
                 }
+                if (isSingleModifier(origKey)) {
+                    inputState->totallyReleased_ = true;
+                }
             }
 
             if (inputState->pendingGroupIndex_ &&
@@ -780,7 +790,7 @@ Instance::Instance(int argc, char **argv) {
                 if (inputState->imChanged_) {
                     inputState->imChanged_->ignore();
                 }
-                d->acceptGroupChange(ic);
+                d->acceptGroupChange(lastKeyPressed, ic);
             }
 
             if (!keyEvent.filtered() && !keyEvent.isRelease()) {
@@ -909,6 +919,18 @@ Instance::Instance(int argc, char **argv) {
                        }
                        engine->keyEvent(*entry, keyEvent);
                    }));
+    d->eventWatchers_.emplace_back(watchEvent(
+        EventType::InputContextVirtualKeyboardEvent,
+        EventWatcherPhase::InputMethod, [this](Event &event) {
+            auto &keyEvent = static_cast<VirtualKeyboardEvent &>(event);
+            auto *ic = keyEvent.inputContext();
+            auto *engine = inputMethodEngine(ic);
+            const auto *entry = inputMethodEntry(ic);
+            if (!engine || !entry) {
+                return;
+            }
+            engine->virtualKeyboardEvent(*entry, keyEvent);
+        }));
     d->eventWatchers_.emplace_back(watchEvent(
         EventType::InputContextInvokeAction, EventWatcherPhase::InputMethod,
         [this](Event &event) {
@@ -973,6 +995,14 @@ Instance::Instance(int argc, char **argv) {
         [this, d](Event &event) {
             auto &icEvent = static_cast<InputContextEvent &>(event);
             activateInputMethod(icEvent);
+
+            if (virtualKeyboardAutoShow()) {
+                auto *inputContext = icEvent.inputContext();
+                if (!inputContext->clientControlVirtualkeyboardShow()) {
+                    inputContext->showVirtualKeyboard();
+                }
+            }
+
             if (!d->globalConfig_.showInputMethodInformationWhenFocusIn() ||
                 icEvent.inputContext()->capabilityFlags().test(
                     CapabilityFlag::Disable)) {
@@ -1010,9 +1040,15 @@ Instance::Instance(int argc, char **argv) {
         }));
     d->eventWatchers_.emplace_back(d->watchEvent(
         EventType::InputContextFocusOut, EventWatcherPhase::InputMethod,
-        [this, d](Event &event) {
+        [this](Event &event) {
             auto &icEvent = static_cast<InputContextEvent &>(event);
             deactivateInputMethod(icEvent);
+            if (virtualKeyboardAutoHide()) {
+                auto *inputContext = icEvent.inputContext();
+                if (!inputContext->clientControlVirtualkeyboardHide()) {
+                    inputContext->hideVirtualKeyboard();
+                }
+            }
         }));
     d->eventWatchers_.emplace_back(d->watchEvent(
         EventType::InputContextReset, EventWatcherPhase::ReservedFirst,
@@ -1111,6 +1147,9 @@ Instance::Instance(int argc, char **argv) {
             auto &icEvent = static_cast<InputContextEvent &>(event);
             d->uiManager_.expire(icEvent.inputContext());
         }));
+    d->eventWatchers_.emplace_back(d->watchEvent(
+        EventType::InputMethodModeChanged, EventWatcherPhase::ReservedFirst,
+        [d](Event &) { d->uiManager_.updateAvailability(); }));
     d->uiUpdateEvent_ = d->eventLoop_.addDeferEvent([d](EventSource *) {
         d->uiManager_.flush();
         return true;
@@ -1344,11 +1383,56 @@ bool Instance::isRunning() const {
     return d->running_;
 }
 
+InputMethodMode Instance::inputMethodMode() const {
+    FCITX_D();
+    return d->inputMethodMode_;
+}
+
+void Instance::setInputMethodMode(InputMethodMode mode) {
+    FCITX_D();
+    if (d->inputMethodMode_ == mode) {
+        return;
+    }
+    d->inputMethodMode_ = mode;
+    postEvent(InputMethodModeChangedEvent());
+}
+
 bool Instance::isRestartRequested() const {
     FCITX_D();
     return d->restart_;
 }
 
+bool Instance::virtualKeyboardAutoShow() const {
+    FCITX_D();
+    return d->virtualKeyboardAutoShow_;
+}
+
+void Instance::setVirtualKeyboardAutoShow(bool autoShow) {
+    FCITX_D();
+    d->virtualKeyboardAutoShow_ = autoShow;
+}
+
+bool Instance::virtualKeyboardAutoHide() const {
+    FCITX_D();
+    return d->virtualKeyboardAutoHide_;
+}
+
+void Instance::setVirtualKeyboardAutoHide(bool autoHide) {
+    FCITX_D();
+    d->virtualKeyboardAutoHide_ = autoHide;
+}
+
+VirtualKeyboardFunctionMode Instance::virtualKeyboardFunctionMode() const {
+    FCITX_D();
+    return d->virtualKeyboardFunctionMode_;
+}
+
+void Instance::setVirtualKeyboardFunctionMode(
+    VirtualKeyboardFunctionMode mode) {
+    FCITX_D();
+    d->virtualKeyboardFunctionMode_ = mode;
+}
+
 InstancePrivate *Instance::privateData() {
     FCITX_D();
     return d;
@@ -1811,6 +1895,9 @@ void Instance::reloadConfig() {
         });
     }
 #endif
+    if (d->running_) {
+        postEvent(GlobalConfigReloadedEvent());
+    }
 }
 
 void Instance::resetInputMethodList() {
@@ -2283,7 +2370,7 @@ void Instance::showInputMethodInformation(InputContext *ic) {
 
 bool Instance::checkUpdate() const {
     FCITX_D();
-    return (d->inFlatpak_ && fs::isreg("/app/.updated")) ||
+    return (isInFlatpak() && fs::isreg("/app/.updated")) ||
            d->addonManager_.checkUpdate() || d->imManager_.checkUpdate() ||
            postEvent(CheckUpdateEvent());
 }
