@@ -224,6 +224,50 @@ QuickPhrase::QuickPhrase(Instance *instance)
                 keyEvent.accept();
                 return;
             }
+            if (keyEvent.key().check(FcitxKey_Delete)) {
+                if (state->buffer_.empty()) {
+                    state->reset(inputContext);
+                } else {
+                    if (state->buffer_.del()) {
+                        if (state->buffer_.empty()) {
+                            state->reset(inputContext);
+                        } else {
+                            updateUI(inputContext);
+                        }
+                    }
+                }
+                keyEvent.accept();
+                return;
+            }
+            if (!state->buffer_.empty()) {
+                const Key &key = keyEvent.key();
+                if (key.check(FcitxKey_Home) || key.check(FcitxKey_KP_Home)) {
+                    state->buffer_.setCursor(0);
+                    keyEvent.accept();
+                    return updateUI(inputContext);
+                } else if (key.check(FcitxKey_End) ||
+                           key.check(FcitxKey_KP_End)) {
+                    state->buffer_.setCursor(state->buffer_.size());
+                    keyEvent.accept();
+                    return updateUI(inputContext);
+                } else if (key.check(FcitxKey_Left) ||
+                           key.check(FcitxKey_KP_Left)) {
+                    auto cursor = state->buffer_.cursor();
+                    if (cursor > 0) {
+                        state->buffer_.setCursor(cursor - 1);
+                    }
+                    keyEvent.accept();
+                    return updateUI(inputContext);
+                } else if (key.check(FcitxKey_Right) ||
+                           key.check(FcitxKey_KP_Right)) {
+                    auto cursor = state->buffer_.cursor();
+                    if (cursor < state->buffer_.size()) {
+                        state->buffer_.setCursor(cursor + 1);
+                    }
+                    keyEvent.accept();
+                    return updateUI(inputContext);
+                }
+            }
             if (!state->typed_ && !state->str_.empty() &&
                 state->buffer_.empty() && keyEvent.key().check(state->key_)) {
                 keyEvent.accept();
@@ -251,6 +295,30 @@ QuickPhrase::QuickPhrase(Instance *instance)
 
             updateUI(inputContext);
         }));
+    eventHandlers_.emplace_back(instance_->watchEvent(
+        EventType::InputContextInvokeAction, EventWatcherPhase::PreInputMethod,
+        [this](Event &event) {
+            auto &invokeActionEvent = static_cast<InvokeActionEvent &>(event);
+            auto *inputContext = invokeActionEvent.inputContext();
+            auto *state = inputContext->propertyFor(&factory_);
+            if (!state->enabled_) {
+                return;
+            }
+            invokeActionEvent.filter();
+            int cursor = invokeActionEvent.cursor() -
+                         static_cast<int>(state->prefix_.size());
+            if (cursor < 0 ||
+                invokeActionEvent.action() !=
+                    InvokeActionEvent::Action::LeftClick ||
+                !inputContext->capabilityFlags().test(
+                    CapabilityFlag::Preedit)) {
+                state->reset(inputContext);
+                return;
+            }
+            state->buffer_.setCursor(cursor);
+            invokeActionEvent.accept();
+            updateUI(inputContext);
+        }));
 
     reloadConfig();
 }
@@ -273,12 +341,8 @@ public:
 
             q_->updateUI(inputContext);
         } else if (action_ == QuickPhraseAction::Commit) {
-            state->reset(inputContext);
-            inputContext->inputPanel().reset();
-            inputContext->updatePreedit();
-            inputContext->updateUserInterface(
-                UserInterfaceComponent::InputPanel, true);
             inputContext->commitString(commit_);
+            state->reset(inputContext);
         }
         // DoNothing and other values are also handled here.
     }
@@ -392,13 +456,13 @@ void QuickPhrase::updateUI(InputContext *inputContext) {
         inputContext->inputPanel().setCandidateList(std::move(candidateList));
     }
     Text preedit;
-    if (!state->prefix_.empty()) {
-        preedit.append(state->prefix_);
-    }
     const bool useClientPreedit =
         inputContext->capabilityFlags().test(CapabilityFlag::Preedit);
     TextFormatFlags format{useClientPreedit ? TextFormatFlag::Underline
                                             : TextFormatFlag::NoFlag};
+    if (!state->prefix_.empty()) {
+        preedit.append(state->prefix_, format);
+    }
     preedit.append(state->buffer_.userInput(), format);
     if (!state->buffer_.empty()) {
         preedit.setCursor(state->prefix_.size() +
@@ -411,7 +475,6 @@ void QuickPhrase::updateUI(InputContext *inputContext) {
     }
     inputContext->inputPanel().setAuxUp(auxUp);
     if (useClientPreedit) {
-        preedit.setCursor(0);
         inputContext->inputPanel().setClientPreedit(preedit);
     } else {
         inputContext->inputPanel().setPreedit(preedit);
