@@ -106,6 +106,7 @@ private:
 Kimpanel::Kimpanel(Instance *instance)
     : instance_(instance), bus_(dbus()->call<IDBusModule::bus>()),
       watcher_(*bus_) {
+    reloadConfig();
     entry_ = watcher_.watchService(
         "org.kde.impanel", [this](const std::string &, const std::string &,
                                   const std::string &newOwner) {
@@ -116,6 +117,8 @@ Kimpanel::Kimpanel(Instance *instance)
 
 Kimpanel::~Kimpanel() {}
 
+void Kimpanel::reloadConfig() { readAsIni(config_, "conf/kimpanel.conf"); }
+
 void Kimpanel::suspend() {
     eventHandlers_.clear();
     proxy_.reset();
@@ -124,6 +127,8 @@ void Kimpanel::suspend() {
     hasRelativeV2_ = false;
 }
 
+const Configuration *Kimpanel::getConfig() const { return &config_; }
+
 void Kimpanel::registerAllProperties(InputContext *ic) {
     std::vector<std::string> props;
     if (!ic) {
@@ -161,9 +166,10 @@ std::string Kimpanel::actionToStatus(Action *action, InputContext *ic) {
     if (action->menu()) {
         type = "menu";
     }
-    return stringutils::concat(
-        "/Fcitx/", action->name(), ":", action->shortText(ic), ":",
-        iconName(action->icon(ic)), ":", action->longText(ic), ":", type);
+    return stringutils::concat("/Fcitx/", action->name(), ":",
+                               action->shortText(ic), ":",
+                               IconTheme::iconName(action->icon(ic)), ":",
+                               action->longText(ic), ":", type);
 }
 
 void Kimpanel::resume() {
@@ -398,12 +404,17 @@ std::string Kimpanel::inputMethodStatus(InputContext *ic) {
     label = extractTextForLabel(label);
 
     static const bool preferSymbolic = !isKDE();
-    if (preferSymbolic && icon == "input-keyboard") {
+    if (*config_.preferTextIcon) {
+        icon = "";
+        altDescription = description;
+        description = label;
+    } else if (preferSymbolic && icon == "input-keyboard") {
         icon = "input-keyboard-symbolic";
     }
 
-    return stringutils::concat("/Fcitx/im:", description, ":", iconName(icon),
-                               ":", altDescription, ":menu,label=", label);
+    return stringutils::concat("/Fcitx/im:", description, ":",
+                               IconTheme::iconName(icon), ":", altDescription,
+                               ":menu,label=", label);
 }
 
 void Kimpanel::updateCurrentInputMethod(InputContext *ic) {
@@ -437,7 +448,7 @@ void Kimpanel::msgV1Handler(dbus::Message &msg) {
                 }
                 menuitems.push_back(stringutils::concat(
                     "/Fcitx/im/", entry->uniqueName(), ":", entry->name(), ":",
-                    iconName(entry->icon()), "::"));
+                    IconTheme::iconName(entry->icon()), "::"));
             }
             proxy_->execMenu(menuitems);
         } else if (stringutils::startsWith(property, "/Fcitx/im/")) {
